@@ -10,7 +10,7 @@ require POSIX;
 
 use vars qw(@ISA $VERSION);
 
-$VERSION = $IO::Tty::VERSION;
+$VERSION = 0.97_01; # keep same as in Tty.pm
 
 @ISA = qw(IO::Handle);
 eval { require IO::Stty };
@@ -82,12 +82,13 @@ sub make_slave_controlling_terminal {
   @_ == 1 or croak 'usage: $pty->make_slave_controlling_terminal();';
 
   my $self = shift;
+  local(*DEVTTY);
 
   # loose controlling terminal explicitely
   if (defined TIOCNOTTY) {
-    if (open (DEVTTY, "/dev/tty")) {
-      ioctl( DEVTTY, TIOCNOTTY, 0 );
-      close DEVTTY;
+    if (open (\*DEVTTY, "/dev/tty")) {
+      ioctl( \*DEVTTY, TIOCNOTTY, 0 );
+      close \*DEVTTY;
     }
   }
 
@@ -96,9 +97,9 @@ sub make_slave_controlling_terminal {
     warn "setsid() failed, strange behavior may result: $!\r\n" if $^W;
   }
 
-  if (open(DEVTTY, "/dev/tty")) {
+  if (open(\*DEVTTY, "/dev/tty")) {
     warn "Could not disconnect from controlling terminal?!\n" if $^W;
-    close DEVTTY;
+    close \*DEVTTY;
   }
 
   # now open slave, this should set it as controlling tty on some systems
@@ -124,15 +125,18 @@ sub make_slave_controlling_terminal {
     }
   }
 
-  if (not open(DEVTTY, "/dev/tty")) {
-    croak "Could not connect pty as controlling terminal!\n";
+  if (not open(\*DEVTTY, "/dev/tty")) {
+    warn "Error: could not connect pty as controlling terminal!\n";
+    return undef;
   } else {
-    close DEVTTY;
+    close \*DEVTTY;
   }
   
   return 1;
 }
 
+*clone_winsize_from = \&IO::Tty::clone_winsize_from;
+*set_raw = \&IO::Tty::set_raw;
 
 1;
 
@@ -144,7 +148,7 @@ IO::Pty - Pseudo TTY object class
 
 =head1 VERSION
 
-0.95_01 BETA
+0.97_01 BETA
 
 =head1 SYNOPSIS
 
@@ -189,37 +193,65 @@ object which is the master side of the pseudo tty.
 
 =over 4
 
-=item ttyname
+=item ttyname()
 
 Returns the name of the slave pseudo tty. On UNIX machines this will
 be the pathname of the device.  Use this name for informational
 purpose only, to get a slave filehandle, use slave().
 
-=item slave
+=item slave()
 
 The C<slave> method will return the slave filehandle of the given
 master pty, opening it anew if necessary.  If IO::Stty is installed,
 you can then call $slave->stty() to modify the terminal settings.
 
-=item close_slave
+=item close_slave()
 
 The slave filehandle will be closed and destroyed.  This is necessary
 in the parent after forking to get rid of the open filehandle,
-otherwise the parent will not notice if the child exits.
+otherwise the parent will not notice if the child exits.  Subsequent
+calls of slave() will return a newly opened slave filehandle.
 
-=item make_slave_controlling_terminal
+=item make_slave_controlling_terminal()
 
 This will set the slave filehandle as the controlling terminal of the
 current process, which will become a session leader, so this should
 only be called by a child process after a fork(), e.g. in the callback
 to sync_exec() (see L<Proc::SyncExec>).
 
+=item set_raw()
+
+Will set the pty to raw.  Note that this is a one-way operation, you
+need IO::Stty to set the terminal settings to anything else.
+
+On some systems, the master pty is not a tty.  This method checks for
+that and returns success anyway on such systems.  Note that this
+method must be called on the slave, and probably should be called on
+the master, just to be sure, i.e.
+
+  $pty->slave->set_raw();
+  $pty->set_raw();
+
+
+=item clone_winsize_from(\*FH)
+
+Gets the terminal size from filehandle FH (which must be a terminal)
+and transfers it to the pty.  Returns true on success and undef on
+failure.  Note that this must be called upon the I<slave>, i.e.
+
+ $pty->slave->clone_winsize_from(\*STDIN);
+
+On some systems, the master pty also isatty.  I actually have no
+idea if setting terminal sizes there is passed through to the slave,
+so if this method is called for a master that is not a tty, it
+silently returns OK.
+
 =back
 
 
 =head1 SEE ALSO
 
-L<IO::Tty>, L<IO::Handle>, L<Expect>, L<Proc::SyncExec>
+L<IO::Tty>, L<IO::Tty::Constant>, L<IO::Handle>, L<Expect>, L<Proc::SyncExec>
 
 
 =head1 MAILING LISTS
@@ -271,5 +303,15 @@ TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 DAMAGE.
 
+In other words: Use at your own risk.  Provided as is.  Your mileage
+may vary.  Read the source, Luke!
+
+And finally, just to be sure:
+
+Any Use of This Product, in Any Manner Whatsoever, Will Increase the
+Amount of Disorder in the Universe. Although No Liability Is Implied
+Herein, the Consumer Is Warned That This Process Will Ultimately Lead
+to the Heat Death of the Universe.
+
 =cut
 
