@@ -1,28 +1,28 @@
 
 use strict;
 $^W = 1; # enable warnings
-use Test;
+my ($testnr, $maxnr, $oknr);
+
+BEGIN { $testnr = 1; $maxnr = 4; print "$testnr..$maxnr\n"; }
+sub ok ($) {
+  if ($_[0]) {
+    print "ok ", $testnr++, "\n";
+    $oknr++;
+  } else {
+    print "not ok ", $testnr++, "\n";
+    my ($package, $filename, $line) = caller;
+    print "# Test failed at $filename line $line.\n";
+  }
+}
+
 use blib;
 use IO::Pty;
 use IO::Tty qw(TIOCSCTTY TIOCNOTTY TCSETCTTY);
 $IO::Tty::DEBUG = 1;
-
-require POSIX;
+use POSIX ();
 
 my $Perl = $^X;
 
-sub set_raw_pty($) {
-  my $ttyno = fileno(shift);
-  my $termios = new POSIX::Termios;
-  $termios->getattr($ttyno) or die "getattr: $!";
-  $termios->setiflag(0);
-  $termios->setoflag(0);
-  $termios->setlflag(0);
-  $termios->setcc(&POSIX::VMIN, 1);
-  $termios->setcc(&POSIX::VTIME, 0);
-  $termios->setattr($ttyno, &POSIX::TCSANOW) or die "setattr: $!";
-}
-
 print "Configuration: $IO::Tty::CONFIG\n";
 print "Checking for appropriate ioctls: ";
 print "TIOCNOTTY " if defined TIOCNOTTY;
@@ -30,7 +30,91 @@ print "TIOCSCTTY " if defined TIOCSCTTY;
 print "TCSETCTTY " if defined TCSETCTTY;
 print "\n";
 
-plan tests => 3;
+print "Checking that returned fd's don't clash with stdin/out/err...\n";
+{
+  my $pid = fork();
+  die "Cannot fork" if not defined $pid;
+  unless ($pid) {
+    # child closes stdin/out and reports test result via exit status
+    sleep 0;
+    close STDIN;
+    close STDOUT;
+    my $master = new IO::Pty;
+    my $slave = $master->slave();
+    if ($master->fileno < 3 or $slave->fileno < 3) {
+      exit(1);
+    }
+    exit(0);
+  }
+
+  die "Wrong child" if wait != $pid;
+  print "child exit=$?\n" if $?;
+  ok($? == 0);
+}
+
+
+print "Checking if child gets pty as controlling terminal...\n";
+{
+  my $master = new IO::Pty;
+
+  pipe(FROM_CHILD, TO_PARENT)
+    or die "Cannot create pipe: $!";
+  my $pid = fork();
+  die "Cannot fork" if not defined $pid;
+  unless ($pid) {
+    # child 
+    $master->make_slave_controlling_terminal();
+    my $slave = $master->slave();
+    close $master;
+    close FROM_CHILD;
+    print TO_PARENT "\n";
+    close TO_PARENT;
+    open(TTY, "+>/dev/tty") or die "no controlling terminal";
+    autoflush TTY 1;
+    print TTY "gimme on /dev/tty: ";
+    my $s = <TTY>;
+    chomp $s;
+    print $slave "back on STDOUT: \U$s\n";
+    close TTY; close $slave;
+    sleep(1);
+    exit 0;
+  }
+
+  close TO_PARENT;
+  $master->close_slave();
+  my $dummy;
+  my $stat = sysread(FROM_CHILD, $dummy, 1);
+  die "Cannot sync with child: $!" if not $stat;
+  close FROM_CHILD;
+
+  my ($s, $chunk);
+  $SIG{ALRM} = sub { ok(0); die "Timeout ($s)"; };
+  alarm(10);
+
+  sysread($master, $s, 100) or die "sysread() failed: $!";
+  ok($s =~ m/gimme.*:/);
+
+  print $master "seems OK!\n";
+
+  # collect all responses
+  my $ret;
+  while ($ret = sysread($master, $chunk, 100)) {
+    $s .= $chunk;
+  }
+  print $s;
+  ok($s =~ m/back on STDOUT: SEEMS OK!/);
+  warn <<"_EOT_" unless defined $ret;
+
+WARNING: when the client closes the slave pty, the master gets an error
+(undef return value and \$! eq "$!")
+instead of EOF (0 return value).  Please be sure to handle this 
+in your application (Expect already does).
+
+_EOT_
+
+  alarm(0);
+  kill TERM => $pid;
+}
 
 # now for the echoback tests
 print <<_EOT_;
@@ -45,8 +129,8 @@ _EOT_
   my $master = new IO::Pty;
   print "isatty(\$master): ", POSIX::isatty($master)? "YES\n": "NO\n";
   if (POSIX::isatty($master)) {
-    eval { set_raw_pty($master); };
-    warn "warning: set_raw_pty(\$master): $@\n" if $@;
+    $master->set_raw() or
+      warn "warning: \$master->set_raw(): $!";
   }
 
   pipe(FROM_CHILD, TO_PARENT)
@@ -59,8 +143,8 @@ _EOT_
     my $slave = $master->slave();
     close $master;
     print "isatty(\$slave): ", POSIX::isatty($slave)? "YES\n": "NO\n";
-    eval { set_raw_pty($slave); };
-    warn "warning: set_raw_pty(\$slave): $@\n" if $@;
+    $slave->set_raw() or
+      warn "warning: \$slave->set_raw(): $!";
     close FROM_CHILD;
     print TO_PARENT "\n";
     close TO_PARENT;
@@ -152,68 +236,6 @@ _EOT_
   kill TERM => $pid;
 }
 
-
-print "Checking if child gets pty as controlling terminal...\n";
-{
-  my $master = new IO::Pty;
-
-  pipe(FROM_CHILD, TO_PARENT)
-    or die "Cannot create pipe: $!";
-  my $pid = fork();
-  die "Cannot fork" if not defined $pid;
-  unless ($pid) {
-    # child 
-    $master->make_slave_controlling_terminal();
-    my $slave = $master->slave();
-    close $master;
-    close FROM_CHILD;
-    print TO_PARENT "\n";
-    close TO_PARENT;
-    open(TTY, "+>/dev/tty") or die "no controlling terminal";
-    autoflush TTY 1;
-    print TTY "gimme on /dev/tty: ";
-    my $s = <TTY>;
-    chomp $s;
-    print $slave "back on STDOUT: \U$s\n";
-    close TTY; close $slave;
-    sleep(1);
-    exit 0;
-  }
-
-  close TO_PARENT;
-  $master->close_slave();
-  my $dummy;
-  my $stat = sysread(FROM_CHILD, $dummy, 1);
-  die "Cannot sync with child: $!" if not $stat;
-  close FROM_CHILD;
-
-  my ($s, $chunk);
-  $SIG{ALRM} = sub { ok(0); die "Timeout ($s)"; };
-  alarm(10);
-
-  sysread($master, $s, 100) or die "sysread() failed: $!";
-  ok($s =~ m/gimme.*:/);
-
-  print $master "seems OK!\n";
-
-  # collect all responses
-  my $ret;
-  while ($ret = sysread($master, $chunk, 100)) {
-    $s .= $chunk;
-  }
-  print $s;
-  ok($s =~ m/back on STDOUT: SEEMS OK!/);
-  warn <<"_EOT_" unless defined $ret;
-
-WARNING: when the client closes the slave pty, the master gets an error
-(undef return value and \$! eq "$!")
-instead of EOF (0 return value).  Please be sure to handle this 
-in your application (Expect already does).
-
-_EOT_
-
-  alarm(0);
-  kill TERM => $pid;
-}
+print "Passed $oknr of $maxnr tests.\n";
 
 
